﻿namespace U2UConsult.Win8.TextBoxSample
{
    using System;
    using System.ComponentModel;
    using System.Text.RegularExpressions;
    using U2UConsult.Win8.TextBoxSample.Common;

    public class MainPageViewModel : BindableBase, IDataErrorInfo
    {
        private string firstName;
        private string lastName;
        private string age;
        private int nbrOfChildren;

        public MainPageViewModel()
        {
            this.PropertyChanged += MainPageViewModel_PropertyChanged;
        }

        public string FirstName
        {
            get { return firstName; }
            set { this.SetProperty(ref firstName, value); }
        }

        public string LastName
        {
            get { return lastName; }
            set { this.SetProperty(ref lastName, value); }
        }

        public string Age
        {
            get { return age; }
            set { this.SetProperty(ref age, value); }
        }

        public int NbrOfChildren
        {
            get { return nbrOfChildren; }
            set { this.SetProperty(ref nbrOfChildren, value); }
        }

        #region Validation

        public string Error
        {
            get
            {
                /// Note from the author: there's an opportunity for refactoring here. I know. Really.

                string error = string.Empty;

                string property = this["FirstName"];
                if (!string.IsNullOrEmpty(property))
                {
                    error += property + Environment.NewLine;
                }

                property = this["LastName"];
                if (!string.IsNullOrEmpty(property))
                {
                    error += property + Environment.NewLine;
                }

                property = this["Age"];
                if (!string.IsNullOrEmpty(property))
                {
                    error += property + Environment.NewLine;
                }

                return error.Trim();
            }
        }

        public string this[string columnName]
        {
            get
            {
                switch (columnName)
                {
                    case "FirstName":
                        if (string.IsNullOrEmpty(firstName))
                        {
                            return "First Name is required.";
                        }

                        return string.Empty;

                    case "LastName":
                        if (string.IsNullOrEmpty(lastName))
                        {
                            return "Last Name is required.";
                        }

                        return string.Empty;

                    case "Age":
                        if (string.IsNullOrEmpty(age))
                        {
                            return "Age is required.";
                        }

                        if (!IsNumeric(age))
                        {
                            return "Age should be numeric.";
                        }

                        return string.Empty;

                    default:
                        return string.Empty;
                }
            }
        }

        public bool HasErrors
        {
            get
            {
                return !string.IsNullOrEmpty(this.Error);
            }
        }

        private void MainPageViewModel_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName != "Error" && e.PropertyName != "HasErrors")
            {
                this.OnPropertyChanged("Error");
                this.OnPropertyChanged("HasErrors");
            }
        }

        // The first I found on the Web. Might need to doublecheck.
        private bool IsNumeric(string value)
        {
            bool match;
            string pattern = "(^[-+]?\\d+(,?\\d*)*\\.?\\d*([Ee][-+]\\d*)?$)|(^[-+]?\\d?(,?\\d*)*\\.\\d+([Ee][-+]\\d*)?$)";
            Regex regEx = new Regex(pattern, RegexOptions.IgnoreCase | RegexOptions.IgnorePatternWhitespace);
            match = regEx.Match(value).Success ? true : false;
            return match;
        }

        #endregion
    }
}
